local Gratuity = AceLibrary("Gratuity-2.0");

-- Detects if a buff is present on the unit and returns the application number
function VisualHeal:DetectBuff(unit, name, app)
    local i=1;
    local state, apps;
    while true do
        _, _, state, apps = UnitBuff(unit, i);
        if (not state) then 
            return false;
        end
        if (string.find(state, name) and ((app == apps) or (app == nil))) then 
            return apps;
        end
        i=i+1;
    end
end

-- Detects if a debuff is present on the unit and returns the application number
function VisualHeal:DetectDebuff(unit, name, app)
    local i=1;
    local state, apps;
    while true do
        _, _, state, apps = UnitDebuff(unit, i);
        if (not state) then 
            return false;
        end
        if (string.find(state, name) and ((app == apps) or (app == nil))) then 
            return apps;
        end
        i=i+1;
    end
end

-- Scan a particular buff/debuff index for buffs contained in tab and returns factor applied to healing
-- returns nil if no buff/debuff at index
-- returns 1.0 if buff does not modify healing
function VisualHeal:ModifierScan(unit, idx, tab, debuff)
    local _, _, icon, apps = (debuff and UnitDebuff or UnitBuff)(unit, idx);
    if (icon) then
        _, _, icon = string.find(icon or "", "Interface\\Icons\\(.+)$");
        local stype = tab[icon..apps] or tab[icon];
        if (stype) then
            if type(stype) == "number" then
                return (debuff and (1.0 - stype) or (1.0 + stype));
            elseif type(stype) == "boolean" then
                if (debuff) then 
                    Gratuity:SetUnitDebuff(unit, idx);
                else 
                    Gratuity:SetUnitBuff(unit, idx) 
                end
                local _, _, modifier = Gratuity:Find(" (%d+)%%", 2, 2, nil, true);
                modifier = tonumber(modifier);
                if (modifier and type(modifier) == "number" and ((modifier >= 0) and (modifier <= 100))) then
                    -- Succesfully scanned and found numerical modifier
                    return (debuff and (1 - modifier / 100) or (1 + modifier / 100));
                else
                    -- Failed in scanning, don't count (de)buff in
                    return 1.0;
                end
            end
        else 
            -- Unknown icon, don't even try to scan
            return 1.0;
        end
    else 
        -- No more buffs/debuff or 
        return nil;
    end
end

-- Tables with known icon names of buffs/debuffs that affect healing
VisualHeal.HealingBuffs = {
    Spell_Shadow_FelArmour     = 0.2   -- Fel Armor
}
    VisualHeal.HealingDebuffs = {
    Ability_CriticalStrike     = true, -- Mortal Wound
    Spell_Shadow_GatherShadows = true, -- Curse of the Deadwood, Veil of Shadow and Gehenna's Curse
    Ability_Warrior_SavageBlow = 0.5,  -- Mortal Strike/Mortal Cleave (Warrior Talent) (app unconfirmed)
    Ability_Rogue_FeignDeath0  = 0.5,  -- Blood Fury (Orc Racial)
    Spell_Shadow_FingerOfDeath = 0.2,  -- Hex of Weakness (app unconfirmed)
    INV_Misc_Head_Dragon_Green = 0.5,  -- Brood Affliction: Green (app unconfirmed)
    Ability_Creature_Poison_03 = true, -- Necrotic Poison 
    Spell_Shadow_LifeDrain     = true, -- Curse of the Bleeding Hollow
    INV_Misc_Herb_16           = true  -- Wound Poison (Stacks 5 times, 10% each)
} 

-- Returns the modifier to healing (as a factor) caused by buffs and debuffs
function VisualHeal:GetHealModifier(unit)
    local healModifier = 1.0;
    for i=1,40 do -- Buffs on unit that affect the amount healed on that unit
        local modifier = self:ModifierScan(unit, i, self.HealingBuffs, false);
        if (modifier) then 
            healModifier = healModifier * modifier;
        else 
            break 
        end
    end 
    for i=1,40 do -- Debuffs on unit that affects the amount healed on that unit
        local modifier = self:ModifierScan(unit, i, self.HealingDebuffs, true);
        if (modifier) then 
            healModifier = healModifier * modifier;
        else 
            break 
        end
    end
    return healModifier;
end

--[[ List buffs and debuffs on a unit
function VisualHeal:ListUnitEffects(unit)
    if UnitExists(unit) then
        local i=1;
        self:Print("|cffffff80******* Buffs on " .. (UnitName(unit) or "Unknown") .. " *******|r");
        while (UnitBuff(unit, i)) do
            local string;
            VisualHeal_ScanningTooltip:SetUnitBuff(unit,i);
            local _, _, icon, apps = UnitBuff(unit, i);
            string = "|cff0080ff" .. (VisualHeal_ScanningTooltipTextLeft1:GetText() or "") .. ":|r|cffffd200 ";
            string = string .. (VisualHeal_ScanningTooltipTextRight1:GetText() or "") .. ", ";
            string = string .. icon .. ", ";
            string = string .. apps .. "|r\n";
            string = string .. ">" .. (VisualHeal_ScanningTooltipTextLeft2:GetText() or "");
            self:Print(string);
            i=i+1;
        end
        i=1;
        self:Print("|cffffff80******* DeBuffs on " .. (UnitName(unit) or "Unknown") .. " *******|r");
        while (UnitDebuff(unit, i)) do
            local string;
            VisualHeal_ScanningTooltip:ClearLines();
            VisualHeal_ScanningTooltip:SetUnitDebuff(unit,i);
            local _, _, icon, apps = UnitDebuff(unit, i);
            string = "|cff0080ff" .. (VisualHeal_ScanningTooltipTextLeft1:GetText() or "") .. ":|r|cffffd200 ";
            string = string .. (VisualHeal_ScanningTooltipTextRight1:GetText() or "") .. ", ";
            string = string .. icon .. ", ";
            string = string .. apps .. "|r\n";
            string = string .. ">" .. (VisualHeal_ScanningTooltipTextLeft2:GetText() or "");
            self:Print(string);
            i=i+1;
        end
    end
end]]
